/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.brooklyn.rest.domain;

import java.io.Serializable;
import java.util.Map;
import java.util.Objects;

import javax.annotation.Nullable;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.annotations.Beta;
import com.google.common.collect.ImmutableMap;

@Beta
public class LinkWithMetadata implements Serializable {

    // TODO remove 'metadata' and promote its contents to be top-level fields; then unmark as Beta

    private static final long serialVersionUID = 3146368899471495143L;

    private final String link;
    private final Map<String,Object> metadata;

    public LinkWithMetadata(
            @JsonProperty("link") String link,
            @Nullable @JsonProperty("metadata") Map<String,?> metadata) {
        this.link = link;
        this.metadata = (metadata == null) ? ImmutableMap.<String,Object>of() : ImmutableMap.<String,Object>copyOf(metadata);
    }

    public String getLink() {
        return link;
    }

    public Map<String, Object> getMetadata() {
        return metadata;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof LinkWithMetadata)) return false;
        LinkWithMetadata that = (LinkWithMetadata) o;
        return Objects.equals(link, that.link) &&
                Objects.equals(metadata, that.metadata);
    }

    @Override
    public int hashCode() {
        return Objects.hash(link, metadata);
    }

    @Override
    public String toString() {
        return "LinkWithMetadata{" +
                "link='" + link + '\'' +
                ", metadata=" + metadata +
                '}';
    }
}
