/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.brooklyn.rest.filter;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.brooklyn.util.text.Identifiers;

/**
 * Tags each request with a probabilistically unique id. Should be included before other
 * filters to make sense.
 */
// TODO Deprecate after porting LoggingFilter
public class RequestTaggingFilter implements Filter {

    private static ThreadLocal<String> tag = new ThreadLocal<String>();

    protected static String getTag() {
        return tag.get();
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        String requestId = Identifiers.makeRandomId(6);
        request.setAttribute(RequestTaggingRsFilter.ATT_REQUEST_ID, requestId);
        tag.set(requestId);
        try {
            chain.doFilter(request, response);
        } finally {
            tag.remove();
        }
    }

    @Override
    public void init(FilterConfig config) throws ServletException {
    }

    @Override
    public void destroy() {
    }

}
