/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.brooklyn.entity.software.base;


import com.google.common.collect.Iterables;
import org.apache.brooklyn.api.location.Location;
import org.apache.brooklyn.location.winrm.WinRmMachineLocation;

public class VanillaWindowsProcessImpl extends SoftwareProcessImpl implements VanillaWindowsProcess {
    @Override
    public Class getDriverInterface() {
        return VanillaWindowsProcessDriver.class;
    }

    @Override
    protected void preStart() {
        super.preStart();
        sensors().set(RDP_PORT, 3389);
        sensors().set(RDP_PORT_CAMEL_CASE, 3389);

        WinRmMachineLocation loc = Iterables.getFirst(Iterables.filter(getLocations(), WinRmMachineLocation.class), null);
        if (loc != null) {
            sensors().set(WINRM_PORT, loc.getPort());
            sensors().set(WINRM_PORT_SHORTEN, loc.getPort());
            sensors().set(WINRM_PORT_CAMEL_CASE, loc.getPort());
        }
    }
    
    @Override
    protected void connectSensors() {
        super.connectSensors();
        connectServiceUpIsRunning();
    }

    @Override
    protected void disconnectSensors() {
        disconnectServiceUpIsRunning();
        super.disconnectSensors();
    }

}
